@@ -5,8 +5,8 @@ use strict;
 use XSLoader ();
 use Exporter ();
 
-our @ISA       = qw(XSLoader Exporter);
-our $VERSION   = '1.02';
+our @ISA       = qw(Exporter);
+our $VERSION   = '1.05';
 our @EXPORT_OK = qw(create);
 
 =head1 NAME
@@ -16,7 +16,10 @@ CDB_File - Perl extension for access to cdb databases
 =head1 SYNOPSIS
 
     use CDB_File;
-    $c = tie %h, 'CDB_File', 'file.cdb' or die "tie failed: $!\n";
+    $c = tie(%h, 'CDB_File', 'file.cdb') or die "tie failed: $!\n";
+
+    # If accessing a utf8 stored CDB_File
+    $c = tie(%h, 'CDB_File', 'file.cdb', utf8 => 1) or die "tie failed: $!\n";
 
     $fh = $c->handle;
     sysseek $fh, $c->datapos, 0 or die ...;
@@ -35,6 +38,8 @@ or
     use CDB_File 'create';
     create %t, $file, "$file.$$";
 
+    # If you want to store the data in utf8 mode.
+    create %t, $file, "$file.$$", utf8 => 1;
 =head1 DESCRIPTION
 
 B<CDB_File> is a module which provides a Perl interface to Dan
@@ -81,6 +86,30 @@ C<$final> containing the contents of C<%t>.  As before,  C<$tmp> must
 name a temporary file which can be atomically renamed to C<$final>.
 C<CDB_File::create> may be imported.
 
+=head2 UTF8 support.
+
+When CDB_File was created in 1997 (prior even to Perl 5.6), Perl SVs
+didn't really deal with UTF8. In order to properly store mixed
+bytes and utf8 data in the file, we would normally need to store a bit
+for each string which clarifies the encoding of the key / values.
+This would be useful since Perl hash keys are downgraded to bytes when
+possible so as to normalize the hash key access regardless of encoding.
+
+The CDB_File format is used outside of Perl and so must maintain file
+format compatibility with those systems. As a result this module provides
+a utf8 mode which must be enabled at database generation and then later
+at read. Keys will always be stored as UTF8 strings which is the opposite
+of how Perl stores the strings. This approach had to be taken to assure no
+data corruption happened due to accidentally downgraded SVs before they
+are stored or on retrieval.
+
+You can enable utf8 mode by passing C<utf8 =E<gt> 1> to B<new>, B<tie>,
+or B<create>. All returned SVs while in this mode will be encoded in utf8.
+This feature is not available below 5.14 due to lack of Perl macro support.
+
+B<NOTE:> read/write of databases not stored in utf8 mode will often be
+incompatible with any non-ascii data.
+
 =head1 EXAMPLES
 
 These are all complete programs.
@@ -316,19 +345,66 @@ fashion, rather than via tie().
 For more information on the methods available on tied hashes see
 L<perltie>.
 
+=head1 THE ALGORITHM
+
+This algorithm is described at L<http://cr.yp.to/cdb/cdb.txt> It is
+small enough that it is included inline in the event that the
+internet loses the page:
+
+=head2 A structure for constant databases
+
+Copyright (c) 1996 D. J. Bernstein, L<djb@pobox.com>
+
+A cdb is an associative array: it maps strings ('keys'') to strings
+('data'').
+
+A cdb contains 256 pointers to linearly probed open hash tables. The
+hash tables contain pointers to (key,data) pairs. A cdb is stored in
+a single file on disk:
+
+    +----------------+---------+-------+-------+-----+---------+
+    | p0 p1 ... p255 | records | hash0 | hash1 | ... | hash255 |
+    +----------------+---------+-------+-------+-----+---------+
+
+Each of the 256 initial pointers states a position and a length. The
+position is the starting byte position of the hash table. The length
+is the number of slots in the hash table.
+
+Records are stored sequentially, without special alignment. A record
+states a key length, a data length, the key, and the data.
+
+Each hash table slot states a hash value and a byte position. If the
+byte position is 0, the slot is empty. Otherwise, the slot points to
+a record whose key has that hash value.
+
+Positions, lengths, and hash values are 32-bit quantities, stored in
+little-endian form in 4 bytes. Thus a cdb must fit into 4 gigabytes.
+
+A record is located as follows. Compute the hash value of the key in
+the record. The hash value modulo 256 is the number of a hash table.
+The hash value divided by 256, modulo the length of that table, is a
+slot number. Probe that slot, the next higher slot, and so on, until
+you find the record or run into an empty slot.
+
+The cdb hash function is C<h = ((h << 5) + h) ^ c>, with a starting
+hash of 5381.
+
+
 =head1 BUGS
 
 The C<create()> interface could be done with C<TIEHASH>.
 
 =head1 SEE ALSO
 
-cdb(3).
+cdb(3)
 
 =head1 AUTHOR
 
 Tim Goodwin, <tjg@star.le.ac.uk>.  B<CDB_File> began on 1997-01-08.
 
-Now maintained by Matt Sergeant, <matt@sergeant.org>
+Work provided through 2008 by Matt Sergeant, <matt@sergeant.org>
+
+Now maintained  by Todd Rinaldo, <toddr@cpan.org>
 
 =cut
 
@@ -340,21 +416,24 @@ sub CLEAR {
 }
 
 sub DELETE {
-    &CLEAR;
+    goto &CLEAR;
 }
 
 sub STORE {
-    &CLEAR;
+    goto &CLEAR;
 }
 
 # Must be preloaded for the prototype.
 
-sub create(\%$$) {
-    my ( $RHdata, $fn, $fntemp ) = @_;
+sub create(\%$$;$$) {
+    my ( $RHdata, $fn, $fntemp, $option_key, $is_utf8 ) = @_;
 
-    my $cdb = CDB_File->new( $fn, $fntemp ) or return undef;
-    my ( $k, $v );
-    $cdb->insert(%$RHdata);
+    die("utf8 CDB_Files are not supported below Perl 5.14") if $option_key && $option_key eq 'utf8' && $is_utf8 && $] < "5.014";
+
+    my $cdb = CDB_File->new( $fn, $fntemp, $option_key || '', $is_utf8 || 0 ) or return undef;
+    {
+        $cdb->insert(%$RHdata);
+    }
     $cdb->finish;
     return 1;
 }
